package com.ruoyi.pt.controller;

import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.pt.service.CasesImportService;
import com.ruoyi.pt.service.RemoteCallsService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

/**
 * 远程推送算法平台数据
 *
 * @author wu
 * @since 2023/12/15 13:41
 */
@Api(tags = "远程推送算法平台数据⼊")
@RestController
@RequestMapping("/remoteCall")
public class RemoteCallsController extends BaseController {

    @Resource
    private RemoteCallsService remoteCallsService;

    @Resource
    private CasesImportService casesImportService;

    /**
     * 获取公共accessToken
     *
     * @return 响应类
     */
    @ApiOperation(value = "获取公共accessToken")
    @GetMapping("/getAccessToken")
    public AjaxResult getAccessToken() {

        return success(remoteCallsService.getAccessToken());
    }

    /**
     * 事件进度推送接口
     *
     * @return 响应类
     */
    @ApiOperation(value = "事件进度推送接口")
    @PostMapping("/imports")
    public AjaxResult imports() {
        remoteCallsService.imports();
        return success();
    }

    /**
     * 获取社会治理中心增量事件保存到events_import表
     *
     * @return 响应类
     */
    @ApiOperation(value = "获取社会治理中心增量事件保存到events_import表")
    @GetMapping("/getEventDate")
    public AjaxResult getEventDate() {
        remoteCallsService.getEventDate();
        return success();
    }

    /**
     * 算法处理所有数据
     *
     * @return 响应类
     */
    @ApiOperation(value = "算法处理所有数据")
    @GetMapping("/urgentAlgorithm")
    public AjaxResult urgentAlgorithm() {
        List<String> list = casesImportService.findDate();
        list.forEach(x -> remoteCallsService.urgentAlgorithm(x));
        return success();
    }
}
