package com.ruoyi.pt.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.pt.entity.CasesImport;
import com.ruoyi.pt.entity.Events;
import com.ruoyi.pt.entity.dto.RemoteCallsDTO;
import com.ruoyi.pt.entity.response.RemoteCallsResponse;
import com.ruoyi.pt.entity.response.RemoteEventResponse;
import com.ruoyi.pt.service.CasesImportService;
import com.ruoyi.pt.service.EventsService;
import com.ruoyi.pt.service.RemoteCallsService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author wu
 * @since 2023/12/15 16:03
 */
@Service(value = "RemoteCallsService")
public class RemoteCallsServiceImpl implements RemoteCallsService {

    public static final Logger log = LoggerFactory.getLogger(RemoteCallsServiceImpl.class);

    @Value("${remote.url}")
    private String url;

    @Value("${remote.appKey}")
    private String appKey;

    @Value("${remote.appSecret}")
    private String appSecret;

    @Value("${remote.grantType}")
    private String grantType;

    @Resource
    private EventsService eventsService;

    @Resource
    private CasesImportService casesImportService;

    /**
     * 获取填充信息后的map
     *
     * @return 填充信息后的map
     */
    private Map<String, Object> getMapValue() {
        Map<String, Object> map = new HashMap<>();
        map.put("appKey", appKey);
        map.put("appSecret", appSecret);
        map.put("grantType", grantType);
        return map;
    }


    /**
     * 获取公共accessToken
     *
     * @return accessToken
     */
    @Override
    public String getAccessToken() {
        Map<String, Object> form = getMapValue();
        String body = HttpRequest.post(url + "/auth/oauth/token").form(form).execute().body();
        RemoteCallsResponse res = JSONUtil.toBean(body, RemoteCallsResponse.class);
        return res.getData().getAccessToken();
    }

    /**
     * 事件进度推送接口
     */
    @Override
    public void imports() {
        String accessToken = this.getAccessToken();
        // 初始化MyBatis-Plus的分页对象 查询第1页，每页100条
        Page<Events> page = new Page<>(1, 100);
        // 创建查询条件
        QueryWrapper<Events> queryWrapper = new QueryWrapper<>();
        // 获取昨天的日期
        LocalDate localDate = LocalDate.now();
        LocalDate yesterday = localDate.minusDays(1);
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        String yesterdayStr = yesterday.format(formatter);
        queryWrapper.isNotNull("innerEventId")
                .and(wrapper -> wrapper.ne("innerEventId", ""))
                .and(wrapper -> wrapper.gt("eventTime", yesterdayStr))
                .and(wrapper -> wrapper
                        .nested(i -> i.eq("scenceType", "3").gt("urgentPercent", 0.994))
                        .or()
                        .ne("scenceType", "3")
                );
        // 创建SimpleDateFormat对象，指定日期格式为yyyy-MM-dd HH:mm:ss
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        // 循环获取所有数据
        while (true) {
            IPage<Events> dataPage = eventsService.page(page, queryWrapper);
            List<Events> data = dataPage.getRecords();
            List<RemoteCallsDTO> dtos = BeanUtil.copyToList(data, RemoteCallsDTO.class);
            dtos.forEach(x -> {
                x.setContent(x.getTitle());
                // 使用SimpleDateFormat对象将Date对象格式化为字符串
                String firstWarnTime = sdf.format(x.getFirstWarnTimeDate());
                String eventTime = sdf.format(x.getEventTimeDate());
                x.setContent(x.getTitle());
                x.setFirstWarnTime(firstWarnTime);
                x.setEventTime(eventTime);
            });
            String jsonStr = JSONUtil.toJsonStr(dtos);
            String body = HttpRequest.post(url + "/gateway/event/event/eventData/imports").header("Authorization", "Bearer " + accessToken).body(jsonStr).execute().body();
            // 判断是否还有下一页
            if (dataPage.getPages() > dataPage.getCurrent()) {
                // 设置下一页的页码
                page.setCurrent(page.getCurrent() + 1);
            } else {
                break; // 已经获取完所有数据，结束循环
            }
        }

    }

    /**
     * 获取社会治理中心增量事件保存到events_import表
     */
    @Override
    public void getEventDate() {
        Long id = casesImportService.findIdDesc();
        if (id == null) {
            id = 0L;
        }
        saveDate(id);
    }

    /**
     * 紧急事件接口
     *
     * @param localDate 昨天日期字符串 yyyy-MM-dd
     */
    @Override
    public void urgentAlgorithm(String localDate) {
        Map<String, Object> body = new HashMap<>();
        body.put("data", localDate);
        String bodyStr = JSONUtil.toJsonStr(body);
        HttpRequest.post("http://localhost:9003/urgent_algorithm").body(bodyStr).execute().body();
        log.info(localDate);
    }

    /**
     * 群体事件接口
     */
    @Override
    public void groupAlgorithm() {
        Map<String, Object> body = new HashMap<>();
        body.put("data", 3);
        String bodyStr = JSONUtil.toJsonStr(body);
        HttpRequest.post("http://localhost:9003/group_algorithm").body(bodyStr).execute().body();
    }

    /**
     * 同人事件接口
     */
    @Override
    public void onepersonAlgorithm() {
        Map<String, Object> body = new HashMap<>();
        body.put("data", 30);
        String bodyStr = JSONUtil.toJsonStr(body);
        HttpRequest.post("http://localhost:9003/oneperson_algorithm").body(bodyStr).execute().body();
    }

    /**
     * 获取社会治理中心增量事件保存到events_import表
     *
     * @param id 社会治理中心增量事件最大id
     */
    public void saveDate(Long id) {
        Map<String, Object> form = new HashMap<>();
        form.put("id", id);
        form.put("size", 5000);
        String body = HttpRequest.get(url + "/event/event/getEventAddListById").form(form).execute().body();
        RemoteEventResponse res = JSONUtil.toBean(body, RemoteEventResponse.class);
        Map<String, Object> map = JSONUtil.parseObj(res.getData());
        Object records = map.get("records");
        String recordsStr = JSONUtil.toJsonStr(records);
        List<CasesImport> list = JSONUtil.toList(recordsStr, CasesImport.class);
        if (CollectionUtil.isNotEmpty(list)) {
            casesImportService.saveBatch(list);
            id = casesImportService.findIdDesc();
            saveDate(id);
        }
    }
}
